# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.consumer.models.m2m import M2MClient, M2MClientWithNextClientSecret
from stytch.core.response_base import ResponseBase


class RotateCancelResponse(ResponseBase):
    """Response type for `Secrets.rotate_cancel`.
    Fields:
      - m2m_client: The M2M Client affected by this operation.
    """  # noqa

    m2m_client: M2MClient


class RotateResponse(ResponseBase):
    """Response type for `Secrets.rotate`.
    Fields:
      - m2m_client: The M2M Client affected by this operation.
    """  # noqa

    m2m_client: M2MClient


class RotateStartResponse(ResponseBase):
    """Response type for `Secrets.rotate_start`.
    Fields:
      - m2m_client: The M2M Client affected by this operation.
    """  # noqa

    m2m_client: M2MClientWithNextClientSecret
